/* 
 * File:   factorizationalgorithmfactory.cpp
 * Copyright 2009 Filippo Bergamasco (fbergama@gmail.com)
 *
 * This file is part of infact.
 *
 * infact is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * infact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with infact.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Created on January 18, 2009, 1:20 PM
 */

#include "includes/factorizationalgorithmfactory.h"
#include "includes/algorithms/trialdivisionfactorizator.h"
#include "includes/algorithms/fermatfactorizator.h"
#include "includes/algorithms/p-1factorizator.h"
#include "includes/algorithms/rhofactorizator.h"
#include "includes/algorithms/ecmfactorizator.h"


#define IT std::vector< FactorizationAlgorithm* >::iterator



FactorizationAlgorithmFactory::FactorizationAlgorithmFactory() {
    algorithms = new std::vector< FactorizationAlgorithm* >();
    algorithms->insert( algorithms->end(), new TrialDivisionFactorizator() );
    algorithms->insert( algorithms->end(), new FermatFactorizator() );
    algorithms->insert( algorithms->end(), new P1Factorizator() );
    algorithms->insert( algorithms->end(), new RhoFactorizator() );
    algorithms->insert( algorithms->end(), new ECMFactorizator() );
}


FactorizationAlgorithmFactory::~FactorizationAlgorithmFactory() {

    if( algorithms != NULL ) {

        for( IT it = algorithms->begin(); it != algorithms->end(); it++)
            delete *it;

        delete algorithms;
        algorithms = NULL;
    }
}

std::string FactorizationAlgorithmFactory::availableAlgorithmsDescriptionString() {
    std::string r = "Available algorithms:\n\n";
    for( IT it = algorithms->begin(); it < algorithms->end(); it++) {
        r +="\t - ";
        r += (*it)->getName();
        r +="\n";
    }
    return r;
}

FactorizationAlgorithm* FactorizationAlgorithmFactory::newInstance( mpz_class* n, std::string name ) {

    for( IT it = algorithms->begin(); it < algorithms->end(); it++) {
        if( name.compare( (*it)->getName() )==0 ) {
            return (*it)->newInstance( n );
        }
    }
    return NULL;
}

std::vector< std::string >* FactorizationAlgorithmFactory::getAlgorithmsNames() {
    std::vector< std::string >* names = new std::vector< std::string >();
    for( IT it = algorithms->begin(); it < algorithms->end(); it++) {
        names->insert( names->end(), (*it)->getName() );
    }
    return names;
}



