
/*
 * File:   fermatfactorizator.cpp
 * Copyright 2009 Filippo Bergamasco (fbergama@gmail.com)
 *
 * This file is part of infact.
 *
 * infact is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * infact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with infact.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Created on January 18, 2009, 1:52 PM
 */

#include "includes/algorithms/fermatfactorizator.h"
#include "includes/infact.h"



FermatFactorizator::~FermatFactorizator() {
}


FactorizationAlgorithm* FermatFactorizator::newInstance( mpz_class* n ) {
    if( infact::vv )
        std::cout <<  "Created new instance of FermatFactorizator" << std::endl;
    return new FermatFactorizator( n );
}


mpz_class* FermatFactorizator::findNonTrivialFactor() {

    // Pre-conditions check
    if( mpz_probab_prime_p(n->get_mpz_t(), 20) ) {
        throw new infact::PrimeFactorizationException();
    }

    mpz_class* two = new mpz_class(2);
    if( mpz_divisible_p( n->get_mpz_t(), two->get_mpz_t() ) ) {
        return two;
    }
    delete two;

    mpz_class* s = new mpz_class( n->get_mpz_t() );
    mpz_class x( n->get_mpz_t() );
    mpz_class y( n->get_mpz_t() );
    mpz_class xsqr( n->get_mpz_t() );
    mpz_sqrt( x.get_mpz_t(), n->get_mpz_t() );
    unsigned short p = 0xFFFF;


    if( infact::vv )
            std::cout << "Checking all numbers between [sqrt(n)...n] " << std::endl;


    while( x < (*n) ) {

        // calc xsqr=x*x
        mpz_mul( xsqr.get_mpz_t(), x.get_mpz_t(), x.get_mpz_t() );

        // calc y=xsqr-N
        mpz_sub( y.get_mpz_t(), xsqr.get_mpz_t(), n->get_mpz_t() );

        if( infact::vv )
            std::cout << "Checking if  y=" << y.get_str(10) << " is a perfect square...";

        if( mpz_perfect_square_p( y.get_mpz_t() ) ) {

            if( infact::vv )
                std::cout << "yes!" << std::endl;

            // calc y
            mpz_sqrt( y.get_mpz_t(), y.get_mpz_t() );
             // calc s
            mpz_sub( s->get_mpz_t(), x.get_mpz_t(), y.get_mpz_t() );

            if( infact::vv )
                std::cout << "Checking if  s=" << s->get_str(10) << " is a non trivial factor...";

            if( (*s) != 1 && (*s) != (*n) ) {
                if( infact::vv )
                    std::cout << "yes!" << std::endl;

                if( infact::progress)
                    std::cout << CLEAR_LINE;

                return s;
            }
            
            if( infact::vv )
                std::cout << "no." << std::endl;
        } else {
            if( infact::vv )
                std::cout << "no." << std::endl;
        }
        
        p--;
        if( infact::progress && p==0 ) {
            mpz_class base(n->get_mpz_t());
            mpz_sqrt( base.get_mpz_t(), n->get_mpz_t() );
            showProgress( &x, &base, n);
            p=0xFFFF;
        }

        x++;
    }

    if( infact::progress)
        std::cout << CLEAR_LINE;

    
    delete s;
    throw new infact::FactorizationFailureException("No (x,y) couple found between sqrt(n) and n.");
}


std::string FermatFactorizator::getName() {
    return "fermat";
}
