/* 
 * File:   mpnumbergenerator.cpp
 * Copyright 2009 Filippo Bergamasco (fbergama@gmail.com)
 *
 * This file is part of infact.
 *
 * infact is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * infact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with infact.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Created on January 16, 2009, 5:58 PM
 */

#include "includes/mpnumbergenerator.h"
#include "includes/infact.h"

using namespace std;


MPNumberGenerator::MPNumberGenerator() {
    long seed;
    cloned=false;
    gmprand = new gmp_randclass(gmp_randinit_default);
    gmprand->seed( (seed=time(NULL)) );

    if( infact::vv )
        cout << "MPNumberGenerator instance initialized with seed: " << seed << endl;
}

MPNumberGenerator::MPNumberGenerator(const MPNumberGenerator& orig) {
    gmprand = orig.gmprand;
    cloned=true;
}

MPNumberGenerator::~MPNumberGenerator() {
    if( !cloned && gmprand!=NULL )
        delete gmprand;
}

mpz_class* MPNumberGenerator::generateRand( mpz_class* max ) {
    return new mpz_class( gmprand->get_z_range( *max ) );
}

mpz_class* MPNumberGenerator::generatePrime( unsigned int nbits, unsigned int attempts ) {
    mpz_class p;
    bool isprime = false;
    int i;

    if( infact::v )
        cout << "Generating an " << nbits << "-bits" << "prime..." << endl;

    if( infact::vv )
        cout << "Attempt\t/\tNum\t/\tPrime?" << endl;
    
    for( i=0; i<attempts; i++ ) {

        p = gmprand->get_z_bits( (unsigned long)nbits );
        isprime = mpz_probab_prime_p(p.get_mpz_t(), 20);

        if( infact::vv )
            cout << " " << i << "\t\t" << p.get_str(10) << "\t\t" << (isprime ? "yes" : "no") << endl;

        if( isprime )
            break;
    }

    if( !isprime ) {
        cout << "Error: Unable to randomize a prime after " << i << " attempts." << endl;
        return NULL;
    }

    if( infact::v )
        cout << "Prime successfully generated after " << i << " attempts with probability 1/(4^20)" << endl;

    return new mpz_class(p);
}


mpz_class* MPNumberGenerator::generateComposite( unsigned int nbits, unsigned int attempts ) {
    mpz_class c;
    int i;
    bool isprime = true;

    if( infact::v )
        cout << "Generating an " << nbits << "-bits" << "composite..." << endl;

    if( infact::vv )
        cout << "Attempt\t/\tNum\t/\tPrime?" << endl;
    
    for( i=0; i<attempts; i++ ) {
        c = gmprand->get_z_bits( (unsigned long)nbits );
        isprime = mpz_probab_prime_p(c.get_mpz_t(), 5);

        if( infact::vv )
            cout << " " << i << "\t\t" << c.get_str(10) << "\t\t" << (isprime ? "yes" : "no") << endl;
        
        if( !isprime )
            break;
    }

    if( isprime ) {
        cout << "Error: Unable to randomize a composite after " << i << " attempts." << endl;
        return NULL;
    }

    if( infact::v )
        cout << "Composite successfully generated after " << i << " attempts." << endl;

    return new mpz_class(c);
}


mpz_class* MPNumberGenerator::generate2pComposite( unsigned int nbits, unsigned int attempts ) {
    mpz_class* p = generatePrime(nbits, attempts);
    mpz_class* q = generatePrime(nbits, attempts);

    if( p!=NULL && q!=NULL ) {
        if( infact::vv )
            cout << endl << "Primes generation ok, multiplying..." << endl;
        mpz_mul( p->get_mpz_t(), p->get_mpz_t(), q->get_mpz_t());
        delete q;
        return p;
    }

    if( p!=NULL)
        delete p;

    if( q!=NULL)
        delete q;

    return NULL;
}


