/*
 * File:   p-1factorizator.cpp
 * Copyright 2009 Filippo Bergamasco (fbergama@gmail.com)
 *
 * This file is part of infact.
 *
 * infact is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * infact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with infact.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Created on January 18, 2009, 1:52 PM
 */

#include "includes/algorithms/p-1factorizator.h"
#include "includes/infact.h"



P1Factorizator::~P1Factorizator() {
}


FactorizationAlgorithm* P1Factorizator::newInstance( mpz_class* n ) {
    if( infact::vv )
        std::cout <<  "Created new instance of P-1Factorizator" << std::endl;
    return new P1Factorizator( n );
}


mpz_class* P1Factorizator::findNonTrivialFactor() {

    // Pre-conditions check
    if( mpz_probab_prime_p(n->get_mpz_t(), 20) ) {
        throw new infact::PrimeFactorizationException();
    }

    mpz_class* two = new mpz_class(2);
    if( mpz_divisible_p( n->get_mpz_t(), two->get_mpz_t() ) ) {
        return two;
    }
    delete two;


    mpz_class B( n->get_mpz_t() );
    mpz_class max( n->get_mpz_t() );
    mpz_class a( n->get_mpz_t());
    mpz_class j( max.get_mpz_t() );

    mpz_class* d = new mpz_class( n->get_mpz_t() );

    
    B=1;
    a=1;
    while( B < max ) {
        if( infact::vv )
            std::cout << "trying p-1 factorization with bound B=" << B.get_str(10) << std::endl;

        a++;
        mpz_powm( a.get_mpz_t(), a.get_mpz_t(), B.get_mpz_t(), n->get_mpz_t() );
        a--;
        
        mpz_gcd( d->get_mpz_t(), a.get_mpz_t(), n->get_mpz_t() );

        if( infact::vv )
            std::cout << "Checking if gcd(a-1, n) is a non trivial factor of n...";

        if( (*d) != 1 && (*d) != (*n) ) {
            if( infact::vv )
                std::cout << "yes!" << std::endl;

            return d;
        }

        if( infact::vv )
                std::cout << "no." << std::endl;

        if( (*d) == (*n) ) {
            delete d;
            throw new infact::FactorizationFailureException("No factor p can be found that satisfies: p-1 | B!:  GCD(a-1,n)=n");
        }
        B++;
    }

    delete d;
    throw new infact::FactorizationFailureException("No factor p can be found that satisfies: p-1 | B!");
   
}


std::string P1Factorizator::getName() {
    return "p-1";
}
