/*
 * File:   rhofactorizator.cpp
 * Copyright 2009 Filippo Bergamasco (fbergama@gmail.com)
 *
 * This file is part of infact.
 *
 * infact is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * infact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with infact.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Created on January 18, 2009, 1:52 PM
 */

#include "includes/algorithms/rhofactorizator.h"
#include "includes/infact.h"



RhoFactorizator::~RhoFactorizator() {
}


FactorizationAlgorithm* RhoFactorizator::newInstance( mpz_class* n ) {
    if( infact::vv )
        std::cout <<  "Created new instance of RhoFactorizator" << std::endl;
    return new RhoFactorizator( n );
}


inline void f( mpz_class* out, mpz_class* x, mpz_class* n ) {
    mpz_mul( out->get_mpz_t(), x->get_mpz_t(), x->get_mpz_t() );
    (*out)++;
    mpz_mod( out->get_mpz_t(), out->get_mpz_t(), n->get_mpz_t() );
}

mpz_class* RhoFactorizator::findNonTrivialFactor( ) {

    // Pre-conditions check
    if( mpz_probab_prime_p( n->get_mpz_t(), 20) ) {
        throw new infact::PrimeFactorizationException();
    }

    mpz_class* two = new mpz_class(2);
    if( mpz_divisible_p( n->get_mpz_t(), two->get_mpz_t() ) ) {
        return two;
    }
    delete two;

    mpz_class x( n->get_mpz_t() );
    mpz_class x2( n->get_mpz_t() );
    x=2;

    f( &x2, &x, n);

    mpz_class* s = new mpz_class(1);
    mpz_sub( s->get_mpz_t(), x.get_mpz_t(), x2.get_mpz_t() );
    mpz_gcd( s->get_mpz_t(), s->get_mpz_t(), n->get_mpz_t() );

    while( (*s) == 1 ) {

        f( &x, &x, n);
        f( &x2,&x2, n );
        f( &x2,&x2, n );

        mpz_sub( s->get_mpz_t(), x.get_mpz_t(), x2.get_mpz_t() );

        if( infact::vv )
            std::cout << "Checking if   1 < GCD( " << s->get_str(10) << " , " << n->get_str(10) << " ) < n" << std::endl;

        mpz_gcd( s->get_mpz_t(), s->get_mpz_t(), n->get_mpz_t() );
    }

    if( (*s) == (*n) ) {
        delete s;
        throw new infact::FactorizationFailureException("Loop completed with no success.");
    }

    if( infact::vv )
            std::cout << "GCD ok! non trivial factor found." << std::endl;

    return s;
}


std::string RhoFactorizator::getName() {
    return "rho";
}
