/* 
 * File:   trialdivisionfactorizator.cpp
 * Copyright 2009 Filippo Bergamasco (fbergama@gmail.com)
 *
 * This file is part of infact.
 *
 * infact is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * infact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with infact.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Created on January 18, 2009, 1:52 PM
 */

#include "includes/algorithms/trialdivisionfactorizator.h"
#include "includes/infact.h"



TrialDivisionFactorizator::~TrialDivisionFactorizator() {
}


FactorizationAlgorithm* TrialDivisionFactorizator::newInstance( mpz_class* n ) {
    if( infact::vv )
        std::cout <<  "Created new instance of TrialDivisionFactorizator" << std::endl;
    return new TrialDivisionFactorizator( n );
}


mpz_class* TrialDivisionFactorizator::findNonTrivialFactor() {
    mpz_class* s = new mpz_class( 2 );
    mpz_class max( n->get_mpz_t() );
    mpz_sqrt( max.get_mpz_t(), n->get_mpz_t() );
    unsigned short p = 0xFFFF;
    
    if( infact::vv )
            std::cout << "Checking all numbers less than " << max.get_str(10) << std::endl;


    while( (*s) <= max ) {

        if( infact::vv )
            std::cout << "Checking if " << n->get_str(10) << " is divisible by " << s->get_str(10) << "...";

        if( mpz_divisible_p( n->get_mpz_t(), s->get_mpz_t()) ) {
            if( infact::vv )
                std::cout << "yes!" << std::endl;
            if( infact::progress)
                std::cout << CLEAR_LINE;
            return s;
        }

        if( infact::vv )
                std::cout << "no." << std::endl;

        p--;
        if( infact::progress && p==0 ) {
            mpz_class zero(0);
            showProgress( s, &zero, &max);
            p=0xFFFF;
        }
        
        (*s)++;
    }

    if( infact::progress)
        std::cout << CLEAR_LINE;

    delete s;
    throw new infact::PrimeFactorizationException();
}


std::string TrialDivisionFactorizator::getName() {
    return "trial-division";
}
